@@ -1,8 +1,20 @@
-# Copyright (c) 2002-2011 Infrae. All rights reserved.
+# -*- coding: utf-8 -*-
+# Copyright (c) 2002-2013 Infrae. All rights reserved.
 # See also LICENSE.txt
-# $Id$
 
 from silva.core.interfaces import IVersionedContent, IVersion
+from zope import interface
+
+
+class IPath(interface.Interface):
+
+    def pathToUrl(path):
+        """Convert a physical path to a URL.
+        """
+
+    def urlToPath(url):
+        """Convert a HTTP URL to a physical path.
+        """
 
 
 class IDocument(IVersionedContent):
