@@ -1,13 +1,13 @@
-# Copyright (c) 2002-2011 Infrae. All rights reserved.
+# -*- coding: utf-8 -*-
+# Copyright (c) 2002-2013 Infrae. All rights reserved.
 # See also LICENSE.txt
-# $Id$
 
 import unittest
 
 from zope.interface.verify import verifyObject
 
 from Products.SilvaDocument.interfaces import IDocument, IDocumentVersion
-from Products.Silva.testing import FunctionalLayer
+from Products.SilvaDocument.testing import FunctionalLayer
 
 
 class DocumentTestCase(unittest.TestCase):
