@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2002-2011 Infrae. All rights reserved.
+# Copyright (c) 2002-2013 Infrae. All rights reserved.
 # See also LICENSE.txt
-# $Id$
 
 import unittest
 from xml.parsers.expat import ExpatError
@@ -9,18 +8,16 @@ from xml.parsers.expat import ExpatError
 from Acquisition import aq_chain
 
 from zope import component
-from zope.publisher.browser import TestRequest
 
+from silva.core.interfaces import IPublicationWorkflow
 from silva.core.references.interfaces import IReferenceService
 from silva.core.references.reference import get_content_id
 
-from Products.Silva.testing import FunctionalLayer, TestCase
-from Products.Silva.tests.helpers import publish_object, open_test_file
+from Products.Silva.testing import TestCase, TestRequest
+from Products.SilvaDocument.testing import FunctionalLayer
 from Products.SilvaDocument.transform import Transformer
 from Products.SilvaDocument.transform.base import Context
 
-TEST_EXTERNAL_LINK_HTML = '<a _silva_href="%s" '\
-    'href="%s" target="_blank" title="">My link</a>'
 TEST_LINK_HTML = '<a _silva_reference="%s" href="reference" ' \
     '_silva_target="%d" target="_blank" title="">My link</a>'
 TEST_DOUBLE_LINK_HTML = '<p>'\
@@ -49,8 +46,8 @@ class KupuTransformerTestCase(TestCase):
         factory.manage_addFolder('folder', 'Folder')
         factory.manage_addFolder('other', 'Other')
         factory.manage_addPublication('publication', 'Publication')
-        factory.manage_addImage(
-            'chocobo', 'Chocobo', file=open_test_file('chocobo.jpg', globals()))
+        with self.layer.open_fixture('chocobo.jpg') as image:
+            factory.manage_addImage('chocobo', 'Chocobo', image)
         self.transformer = Transformer.EditorTransformer(editor='kupu')
         # Context need the a document version in order to determine
         # references, and REQUEST to compute link URLs
@@ -132,36 +129,6 @@ class KupuTransformerTestCase(TestCase):
             sourceobj=result, context=self.context).asBytes('utf-8')
         self.assertEqual(roundtrip, html)
 
-    def test_regular_external_link_round_trip(self):
-        """Save a regular link.
-        """
-        url = "http://infrae.com/"
-        html = TEST_EXTERNAL_LINK_HTML % (url, url)
-        result = self.transformer.to_source(
-            targetobj=html, context=self.context).asBytes('utf-8')
-        self.assertEqual(
-            result,
-            '<link url="%s" target="_blank" title="">My link</link>' % url)
-
-        roundtrip = self.transformer.to_target(
-            sourceobj=result, context=self.context).asBytes('utf-8')
-        self.assertEqual(roundtrip, html)
-
-    def test_unicode_external_link_round_trip(self):
-        """Save a regular link.
-        """
-        url = u"http://infrae.com/élémentaire".encode('utf-8')
-        html = TEST_EXTERNAL_LINK_HTML % (url, url)
-        result = self.transformer.to_source(
-            targetobj=html, context=self.context).asBytes('utf-8')
-        self.assertEqual(
-            result,
-            '<link url="%s" target="_blank" title="">My link</link>' % url)
-
-        roundtrip = self.transformer.to_target(
-            sourceobj=result, context=self.context).asBytes('utf-8')
-        self.assertEqual(roundtrip, html)
-
     def test_new_image_round_trip(self):
         """We create a new image which is a reference to an image by
         transforming Kupu->Silva, and we check the result by
@@ -492,7 +459,7 @@ class KupuTransformerTestCase(TestCase):
         reference.add_tag(reference_name)
 
         # Step two, make a new copy
-        publish_object(self.root.document)
+        IPublicationWorkflow(self.root.document).publish()
         self.root.document.create_copy()
         new_version = self.root.document.get_editable()
         self.failIfEqual(version, new_version)
@@ -503,7 +470,7 @@ class KupuTransformerTestCase(TestCase):
         self.assertEqual(
             list(service.get_references_to(self.root.publication)),
             [])
-        self.assertListEqual(
+        self.assertItemsEqual(
             list(service.get_references_to(self.root.folder)),
             [reference, new_reference])
         self.assertEqual(
