@@ -1,3 +1,6 @@
+# -*- coding: utf-8 -*-
+# Copyright (c) 2012-2013 Infrae. All rights reserved.
+# See also LICENSE.txt
 """
 module for conversion from current
 
@@ -31,9 +34,9 @@ from urlparse import urlparse
 
 from zExceptions import NotFound
 
-from Products.Silva.mangle import Path
 from Products.SilvaDocument.transform.base import Element, Text, Frag
-from silva.core.interfaces import IPath, IImage
+from silva.core.interfaces import IImage
+from Products.SilvaDocument.interfaces import IPath
 from silva.core.references.reference import get_content_from_id
 
 import silvaformat
@@ -44,6 +47,17 @@ DEBUG=0
 TOPLEVEL = ['h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'h7', 'p', 'pre', 'table', 'img', 'ul', 'ol', 'dl', 'div']
 CONTAINERS = ['body', 'td', 'li', 'th'] # XXX should only contain li's that are descendant of nlist
 
+def Path(base_path, item_path):
+    """mangle path"""
+    i = 0
+    absolute = 0
+    for i in range(0, min(len(item_path), len(base_path))):
+        if item_path[i] != base_path[i]:
+            absolute = 1
+            break
+    if not absolute:
+        item_path = item_path[len(base_path):]
+    return item_path
 
 def is_table_or_source(node):
     """tell you if the node is a table or a code source.
