@@ -1,3 +1,6 @@
+# -*- coding: utf-8 -*-
+# Copyright (c) 2012-2013 Infrae. All rights reserved.
+# See also LICENSE.txt
 """
 module for conversion from current silva (0.9.1) XML to
 kupu (HEAD) version HTML.
@@ -14,18 +17,17 @@ __author__='holger krekel <hpk@trillke.net>'
 __version__='$Revision: 1.26 $'
 
 import operator
-import logging
 
 from Products.SilvaDocument.externalsource import getSourceForId
+from Products.SilvaDocument.interfaces import IPath
 from Products.SilvaDocument.transform.base import Element, Frag, Text
-from silva.core.interfaces import IPath, IImage
+from silva.core.interfaces import IImage
 from silva.core.views.interfaces import IVirtualSite
 from zope.traversing.browser import absoluteURL
 
 
 import htmlformat
 html = htmlformat
-logger = logging.getLogger('Products.SilvaDocument')
 
 from silva.translations import translate as _
 
@@ -278,20 +280,13 @@ class link(SilvaElement):
             reference_name = str(self.getattr('reference'))
             reference_name, reference = context.get_reference(
                 reference_name, read_only=True)
-            if reference is None:
-                logger.error(
-                    u"Invalid reference link detected in document: %s" %
-                    context.model.absolute_url())
-                target_id = 0
-                reference_name = 'new'
-            else:
-                target_id = reference.target_id
+            assert reference is not None, "Invalid reference"
             attributes = {
                 'href': 'reference',
                 'title': title,
                 'target': target,
                 '_silva_anchor': anchor,
-                '_silva_target': target_id,
+                '_silva_target': reference.target_id,
                 '_silva_reference': reference_name}
             if not reference.target_id:
                 attributes['class'] = 'broken-link'
@@ -301,9 +296,8 @@ class link(SilvaElement):
 
         path = ''
         if self.hasattr('url'):
-            # URL can have unicode characters
-            url = unicode(self.getattr('url'))
-            path = IPath(context.request).pathToUrlPath(url)
+            url = self.getattr('url')
+            path = IPath(context.request).pathToUrlPath(str(url))
         return html.a(
             self.content.convert(context),
             href=path,
@@ -339,35 +333,25 @@ class image(SilvaElement):
         attributes = {'alignment': self.getattr('alignment', 'default'),
                       'alt': self.getattr('title', '')}
 
-        def broken_image():
-            site = IVirtualSite(context.request)
-            attributes['src'] = site.get_root_url() + \
-                "/++resource++Products.SilvaDocument/broken-link.jpg"
-            attributes['alt'] = u'Referenced image is missing.'
-
         if self.hasattr('reference'):
             # We have a reference
             reference_name = str(self.getattr('reference'))
             reference_name, reference = context.get_reference(
                 reference_name, read_only=True)
-            if reference is None:
-                logger.error(
-                    u"Invalid image reference detected in document: %s",
-                    context.model.absolute_url())
-                attributes['_silva_target'] = '0'
-                attributes['_silva_reference'] = 'new'
-                broken_image()
+            assert reference is not None, "Invalid reference"
+            attributes['_silva_target'] = reference.target_id
+            attributes['_silva_reference'] = reference_name
+            image = reference.target
+            if image is not None:
+                attributes['src'] = absoluteURL(image, context.request)
             else:
-                attributes['_silva_target'] = reference.target_id
-                attributes['_silva_reference'] = reference_name
-                image = reference.target
-                if image is not None:
-                    attributes['src'] = absoluteURL(image, context.request)
-                else:
-                    broken_image()
+                site = IVirtualSite(context.request)
+                attributes['src'] = site.get_root_url() + \
+                    "/++resource++Products.SilvaDocument/broken-link.jpg"
+                attributes['alt'] = u'Referenced image is missing.'
         elif self.hasattr('path'):
-            path = unicode(self.getattr('path'))
-            src = IPath(context.request).pathToUrlPath(path)
+            path = self.getattr('path')
+            src = IPath(context.request).pathToUrlPath(str(path))
             attributes['src'] = src
             try:
                 image = context.model.unrestrictedTraverse(src.split('/'))
@@ -378,7 +362,7 @@ class image(SilvaElement):
 
         if image and IImage.providedBy(image):
             attributes['width'], attributes['height'] = \
-                image.getDimensions(image.image)
+                image.get_dimensions(image.image)
 
         return html.img(self.content.convert(context), **attributes)
 
@@ -426,10 +410,9 @@ class table(SilvaElement):
     def compute_aligns_relwidths(self):
         """ return a list with the alignments """
         infos = str(self.attr.column_info).split(' ')
-        mapping = self.alignmapping
-        aligns = [
-            (mapping[i[0]] if i[0] in mapping else 'left')
-            for i in infos if i]
+        aligns = [(self.alignmapping.has_key(i[0]) and
+                        self.alignmapping[i[0]] or 'left')
+                    for i in infos]
         try:
             relwidths = [int(i[2:]) for i in infos]
         except ValueError:
